///////////////////////////////////////////////////////////////////////////////
// Sample 0
// Base Sample
//
///////////////////////////////////////////////////////////////////////////////

#include "sample0.h"

DECLARE_TRACE;

///INIT//////////////////////////////////////////////////////////
void init ( void )
{
	std::string temp = "Gfx\\OpenGL.bmp";

   START_TRACE; //readies the debug file

   //Initiating the screen
	SampleScreen.Init();
   SampleScreen.SetFont(GLUT_BITMAP_TIMES_ROMAN_24);
   MYTRACE("Screen is initiated.\n");

   //Loading the OpenGL logo
   LogoID = SampleScreen.GetRessSize(LOAD_TXT);
   SampleScreen.SetTargetType(TXT_RGB); //Target texture type
  	SampleScreen.SetTranspColor(0,0,0); //transparent color in the texture
   Running = SampleScreen.Load(LOAD_TXT, temp); //"Gfx\\OpenGL.bmp");
   if (Running == false)
   	{
   	MYTRACE("Error in loading the OpenGL logo.");
      return;
      }
   MYTRACE("OpenGL logo loaded.\n");

   //Setting up texture coords for the OpenGL logo
   GLLogoTexCoords[0] = Frm::Set2(0.0f, 0.0f);
   GLLogoTexCoords[1] = Frm::Set2(1.0f, 0.0f);
   GLLogoTexCoords[2] = Frm::Set2(0.0f, 1.0f);
   GLLogoTexCoords[3] = Frm::Set2(1.0f, 1.0f);

   //Initiating the windows timer
   SampleTimer->InitTimer(0);
   Elapsed = 0;
   MYTRACE("Timer instance opened.\n");

   printf("Ready.");
}

///DRAWLOGO///////////////////////////////////////////////////////
void drawlogo(void)
{
	glDisable(GL_DEPTH_TEST);
	SampleScreen.SetColor(255, 255, 255);
   SampleScreen.SelectTexture(LogoID);
   glBegin(GL_QUADS);
   	glTexCoord2fv(GLLogoTexCoords[0].data); glVertex3fv(GLLogoQuad[0].data);
      glTexCoord2fv(GLLogoTexCoords[1].data); glVertex3fv(GLLogoQuad[1].data);
      glTexCoord2fv(GLLogoTexCoords[3].data); glVertex3fv(GLLogoQuad[3].data);
      glTexCoord2fv(GLLogoTexCoords[2].data); glVertex3fv(GLLogoQuad[2].data);
   glEnd();
   glEnable(GL_DEPTH_TEST);
}

///DISPLAY////////////////////////////////////////////////////////
void display ( void )
{
	if(Running == false)
   	return;
	SampleTimer->GetTime(0);//Reset the main timer
	SampleScreen.Start_Draw();//Clear the screen

   SampleScreen.SetColor(255, 255, 255);

   SampleScreen.TwoD();//Setup 2D coordinate System
   SampleScreen.PrintFPS(Elapsed); //calculate and display the FPS
   drawlogo();

   SampleScreen.ThreeD(); //Setup 3D coordinate System


   SampleScreen.End_Draw();//Swap buffers
   Elapsed = SampleTimer->GetTime(0); //calculate the elapsed time
}

///CENTERLOGO//////////////////////////////////////////////////////
void centerlogo(void)
{
   //Calculating the OpenGL logo vertices to center on screen
   float sx = (float)((glutGet(GLUT_WINDOW_WIDTH)-512)>>1);
   float sy = (float)((glutGet(GLUT_WINDOW_HEIGHT)-256)>>1);
   GLLogoQuad[0] = Frm::Set3(sx, sy, 0.0f);
   GLLogoQuad[1] = Frm::Set3(sx + 512.0f, sy, 0.0f);
   GLLogoQuad[2] = Frm::Set3(sx, sy + 256.0f, 0.0f);
   GLLogoQuad[3] = Frm::Set3(sx + 512.0f, sy + 256.0f, 0.0f);
}

///RESHAPE/////////////////////////////////////////////////////////
void reshape(int pw, int ph)
{
	SampleScreen.Shape(pw, ph);
	centerlogo();
}

///IDLE//////////////////////////////////////////////////////////
void idle ( void )
{
	if (Running == false)
		return;

	glutPostRedisplay ( ); //Tell Glut to show the result
}

///KEYBOARD////////////////////////////////////////////////////////
#pragma argsused
void keyboard ( unsigned char pkey, int px, int py )
{
   switch ( pkey ) {
      case 27:  /*  Escape key  */
         Running = false;
	      //this is the end, my only friend ...
   	   exit(0);
         break;
      case 'f':
         glutFullScreen ( );
         centerlogo();
         break;
      case 'w':
      	//switch back from full screen
         glutReshapeWindow ( SCREENWIDTH,SCREENHEIGHT );
         break;
      default:
         break;
   }
}

#pragma argsused
void keyboard2 ( int pkey, int px, int py )
{
}

///EXITFUNC////////////////////////////////////////////////////////
void exitfunc(void)
{
	MYTRACE("Closing ...");
   //we unload all textures from OpenGL
   SampleScreen.Unload(LOAD_TXT, 0);
   //we delete the timer instance
   delete SampleTimer;
   END_TRACE;
}

/*  Main Loop
 *  Open window with initial window size, title bar,
 *  RGBA display mode, and handle input events.
 */
int main ( int argc, char** argv )
{
   glutInit ( &argc, argv );
   glutInitDisplayMode ( GLUT_RGB | GLUT_DOUBLE );
   glutInitWindowPosition( SCREENPOSX, SCREENPOSY );
   glutInitWindowSize  ( SCREENWIDTH, SCREENHEIGHT );
   glutCreateWindow    ( argv[0]  );
   atexit(exitfunc);
   init ();
   glutReshapeFunc     ( reshape  );
   glutKeyboardFunc    ( keyboard );
   glutSpecialFunc	  ( keyboard2);
   glutDisplayFunc     ( display  );
   glutIdleFunc 		  ( idle  );
   glutMainLoop        ( );
   return 0;
}
