/////////////////////////////////////////////////////////
// Manager_Texture.cpp
// Texture Manager loading code
//
/////////////////////////////////////////////////////////

#include "GL\Manager_Texture.h"

Manager_Texture::~Manager_Texture(void)
{
	while(!_Ts.empty())
   	{
      glDeleteTextures(1, &(_Ts.back()->ID));
      delete _Ts.back();
      _Ts.pop_back();
      }
}

//MANAGER_TEXTURE////////////////////////////////////////
// LOAD /////////////////////////////////////////////////
// Loads a texture and pushes the GL handle to the
// specialized manager
/////////////////////////////////////////////////////////
bool Manager_Texture::Load(std::string &pFileName)
{
	if (!check(pFileName))
   	return false;

   if (!IOStrategy->Load(pFileName, _Texture))
   	return false;

   if (_Texture.Type != _TargetType)
   	_Convert.Convert(&_Texture, _TargetType);

   _TextStruct = new Texture;
   _TextStruct->Name = pFileName;
   _TextStruct->TargetType = _TargetType;

   if (!Create())
   	{
      delete _TextStruct;
   	return false;
      }

   _Ts.push_back(_TextStruct);
	return true;
}

//MANAGER_TEXTURE////////////////////////////////////////
// UNLOAD ///////////////////////////////////////////////
// Unloads all textures after pIndex
//
/////////////////////////////////////////////////////////
bool Manager_Texture::Unload(uint16 pIndex)
{
	if (_Ts.size() <= pIndex)
   	return false;
	while(_Ts.size()>pIndex)
   	{
      glDeleteTextures(1, &(_Ts.back()->ID));
      delete _Ts.back();
      _Ts.pop_back();
      }
   return true;
}

//MANAGER_TEXTURE////////////////////////////////////////
// RESET ////////////////////////////////////////////////
// Reloads all textures
//
/////////////////////////////////////////////////////////
void Manager_Texture::Reset(void)
{
	for (std::vector<Texture*>::iterator i = _Ts.begin(); i!= _Ts.end(); i++)
		{
      _TextStruct = *i;
      glDeleteTextures(1, &_TextStruct->ID);
      IOStrategy->Load(_TextStruct->Name, _Texture);

      _TargetType = _TextStruct->TargetType;

   	if (_Texture.Type != _TargetType)
   		_Convert.Convert(&_Texture, _TargetType);

   	Create();
      }
}

//MANAGER_TEXTURE////////////////////////////////////////
// SETTEXTURE ///////////////////////////////////////////
// Checks the texture name. If it exits, return the Index
// else create the texture the return an Index
/////////////////////////////////////////////////////////
uint16 Manager_Texture::SetTexture(std::string &pFileName)
{
	for (uint16 i = 0; i< _Ts.size(); i++)
      if (strcmp(pFileName.c_str(), _Ts[i]->Name.c_str()) == 0)
      	return i;

   if (!Load(pFileName))
   	return TXTMGR_NOTEXTURE;

   return (_Ts.size()-1);
}

//MANAGER_TEXTURE////////////////////////////////////////
// CHECK ////////////////////////////////////////////////
// Checks the file type to open the correct strategy
//
/////////////////////////////////////////////////////////
bool Manager_Texture::check(std::string &pFileName)
{
	long pos;

   pos = (pFileName.find(".BMP")!=-1) ? pFileName.find(".BMP") : pFileName.find(".bmp");
   if (pos > -1)
   	{
      if ((IOStrategy == 0) || (IOStrategy->_Type != IO_BMP))
      	{
         delete IOStrategy;
         IOStrategy = (Frm::IO<TEXTURE>*)new ToolBox::IO_Texture_BMP;
         }/**/
  		return true;
      }

	return false;
}

//MANAGER_TEXTURE////////////////////////////////////////
// CREATE ///////////////////////////////////////////////
// Creates the texture in OpenGL
//
/////////////////////////////////////////////////////////
bool Manager_Texture::Create(void)
{
   GLuint TextID;

   glGenTextures(1, &TextID); //affects an unused name of texture object
   glBindTexture(GL_TEXTURE_2D, TextID);    // Pointe la texture

   glPixelStorei(GL_UNPACK_ALIGNMENT,1);        // Construit la texture sous forme de donnes
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // Linear Filtering
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // Linear Filtering
   glTexEnvf ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

   // Generate The Texture
   switch (_Texture.Type) {
   	case TXT_RGB:
  			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _Texture.Width, _Texture.Height, 0, GL_RGB, GL_UNSIGNED_BYTE, _Texture.Bits);
      	break;
      case TXT_RGBA:
      	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, _Texture.Width, _Texture.Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, _Texture.Bits);
         break;
      case TXT_ALPHA:
      	glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, _Texture.Width, _Texture.Height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, _Texture.Bits);
         break;
      case TXT_LA:
      	glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE_ALPHA, _Texture.Width, _Texture.Height, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, _Texture.Bits);
   	}

   _TextStruct->ID = TextID;

   delete[] _Texture.Bits;

   return true;
}/**/
